@@ -1,17 +1,16 @@
-import os
 import pathlib
 
 from pycape import Cape
-from pycape import FunctionRef
+
+token_file = pathlib.Path(__file__).parent.absolute() / "user.token"
+
+cape = Cape()
+function_ref = cape.function("pycape-dev/echo")
+token = cape.token(token_file)
+
 
 if __name__ == "__main__":
-    url = os.environ.get("CAPE_HOST", "https://app.capeprivacy.com")
-    function_json = os.environ.get("FUNCTION_JSON", "echo_token.json")
-    function_json = pathlib.Path(__file__).parent.absolute() / function_json
-
-    function_ref = FunctionRef.from_json(function_json)
-    cape = Cape(url=url)
-    cape.connect(function_ref)
+    cape.connect(function_ref, token)
     result = cape.invoke("Hello Cape".encode())
     print(f"The result is: {result.decode()}")
     result = cape.invoke("Hello Gavin".encode())
@@ -23,7 +22,7 @@ if __name__ == "__main__":
     # Note that instead you can use the connection_context
     # context manager which will automatically close the
     # connection and reset websocket connection's states.
-    with cape.function_context(function_ref):
+    with cape.function_context(function_ref, token):
         result = cape.invoke("Hello Cape".encode())
         print(f"The result is: {result.decode()}")
         result = cape.invoke("Hello Gavin".encode())
