@@ -1,9 +1,11 @@
 from pycape.cape import Cape
 from pycape.function_ref import FunctionRef
+from pycape.token import Token
 
-__version__ = "2.1.0"
+__version__ = "3.0.0"
 
 __all__ = [
     "Cape",
     "FunctionRef",
+    "Token",
 ]
