@@ -1,9 +1,7 @@
 import base64
 import json
 import unittest
-from unittest.mock import Mock
 
-from pycape import function_ref as fref
 from pycape.cape import _create_connection_request
 from pycape.cape import _generate_nonce
 from pycape.cape import _handle_expected_field
@@ -37,23 +35,6 @@ class TestCape(unittest.TestCase):
         inner_msg = _parse_wss_response(response)
         self.assertEqual(inner_msg, base64.b64decode("conn"))
 
-    def test_connect(self):
-        Cape = Mock()
-        client = Cape()
-        function_ref = fref.FunctionRef("mHwsZ9Bh5cK4Bz8utHjdhy", "my_token")
-        client.connect(function_ref)
-        client.connect.assert_called_with(function_ref)
-
-    def test_run(self):
-        Cape = Mock()
-        client = Cape()
-        function_ref = fref.FunctionRef("mHwsZ9Bh5cK4Bz8utHjdhy", "my_token")
-        input = "Welcome to Cape"
-        result = client.run(
-            function_ref, input.encode()
-        ).return_value = b"Welcome to Cape"
-        self.assertEqual(result.decode(), input)
-
 
 if __name__ == "__main__":
     unittest.main()
